%define anolis_release 3
#defining macros needed by SELinux
%global selinuxtype targeted
%global modulename vncsession

Name:           tigervnc
Version:        1.13.1
Release:        %{anolis_release}%{?dist}
Summary:        A TigerVNC remote display system

%global _hardened_build 1

License:        GPLv2+
URL:            http://www.tigervnc.com

Source0:        https://github.com/TigerVNC/%{name}/archive/v%{version}.tar.gz#/%{name}-%{version}.tar.gz
Source1:        xvnc.service
Source2:        xvnc.socket
Source3:        10-libvnc.conf
Source4:        HOWTO.md

# Backwards compatibility
Source5:        vncserver

# Downstream patches

# Upstream patches
Patch50:        tigervnc-vncsession-restore-script-systemd-service.patch

# This is tigervnc-%%{version}/unix/xserver116.patch rebased on the latest xorg
Patch100:       tigervnc-xserver120.patch
Patch101:       101-bugfix-for-CVE-2024-21885.patch

BuildRequires:  make
BuildRequires:  gcc-c++
BuildRequires:  automake, autoconf, libtool, gettext, gettext-autopoint
BuildRequires:  cmake, desktop-file-utils
BuildRequires:  libxkbfile-devel, openssl-devel, libpciaccess-devel
BuildRequires:  freetype-devel, libjpeg-turbo-devel, gnutls-devel, pam-devel
# X11/graphics dependencies
BuildRequires: xorg-x11-server-source
BuildRequires: libXext-devel, libX11-devel, libXi-devel, libXfixes-devel
BuildRequires: libXdamage-devel, libXrandr-devel, libXt-devel, libXdmcp-devel
BuildRequires: libXinerama-devel, mesa-libGL-devel, libxshmfence-devel
BuildRequires: pixman-devel, libdrm-devel
BuildRequires: xorg-x11-util-macros, xorg-x11-xtrans-devel, libXtst-devel
BuildRequires: xorg-x11-font-utils
BuildRequires:  libXfont2-devel
BuildRequires:  libappstream-glib
# SELinux
BuildRequires:  libselinux-devel, selinux-policy-devel, systemd

# TigerVNC 1.4.x requires fltk 1.3.3 for keyboard handling support
# See https://github.com/TigerVNC/tigervnc/issues/8, also bug #1208814
BuildRequires:  fltk-devel >= 1.3.3
BuildRequires:  xorg-x11-server-devel
BuildRequires:  libXrender-devel

Requires(post): coreutils
Requires(postun):coreutils

Requires:       hicolor-icon-theme
Requires:       tigervnc-license
Requires:       tigervnc-icons

%description
Virtual Network Computing (VNC) is a remote display system which
allows you to view a computing 'desktop' environment not only on the
machine where it is running, but from anywhere on the Internet and
from a wide variety of machine architectures.  This package contains a
client which will allow you to connect to other desktops running a VNC
server.

%package doc
Summary:        Documentation files for %{name}
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentation files for %{name}.

%package server
Summary:        A TigerVNC server
Requires:       perl-interpreter
Requires:       tigervnc-server-minimal = %{version}-%{release}
Requires:       (%{name}-selinux if selinux-policy-%{selinuxtype})
Requires:       xorg-x11-xauth
Requires:       xorg-x11-xinit
Requires(post): systemd
Requires(preun): systemd
Requires(postun): systemd
Requires(post): systemd

%description server
The VNC system allows you to access the same desktop from a wide
variety of platforms.  This package includes set of utilities
which make usage of TigerVNC server more user friendly. It also
contains x0vncserver program which can export your active
X session.

%package server-minimal
Summary:        A minimal installation of TigerVNC server
Requires(post): systemd
Requires(preun): systemd
Requires(postun): systemd
Requires(post): systemd

Requires:       mesa-dri-drivers, xkeyboard-config, xkbcomp
Requires:       tigervnc-license, dbus-x11

%description server-minimal
The VNC system allows you to access the same desktop from a wide
variety of platforms. This package contains minimal installation
of TigerVNC server, allowing others to access the desktop on your
machine.

%package server-module
Summary:        TigerVNC module to Xorg
Requires:       xorg-x11-server-Xorg %(xserver-sdk-abi-requires ansic) %(xserver-sdk-abi-requires videodrv)
Requires:       tigervnc-license

%description server-module
This package contains libvnc.so module to X server, allowing others
to access the desktop on your machine.

%package license
Summary:        License of TigerVNC suite
BuildArch:      noarch

%description license
This package contains license of the TigerVNC suite

%package icons
Summary:        Icons for TigerVNC viewer
BuildArch:      noarch

%description icons
This package contains icons for TigerVNC viewer

%package selinux
Summary:        SELinux module for TigerVNC
BuildArch:      noarch
BuildRequires:  selinux-policy-devel
Requires:       selinux-policy-%{selinuxtype}
Requires(post): selinux-policy-%{selinuxtype}
BuildRequires:  selinux-policy-devel
%{?selinux_requires}

%description selinux
This package provides the SELinux policy module to ensure TigerVNC
runs properly under an environment with SELinux enabled.

%prep
%setup -q

%patch50 -p1 -b .vncsession-restore-script-systemd-service

cp -r /usr/share/xorg-x11-server-source/* unix/xserver
pushd unix/xserver
for all in `find . -type f -perm -001`; do
        chmod -x "$all"
done
%patch100 -p1 -b .xserver120-rebased
%patch101 -p1 -b .101-bugfix-for-CVE-2024-21885
popd

# Downstream patches

%build
export CFLAGS="$RPM_OPT_FLAGS -fpic"
export CXXFLAGS="$CFLAGS -std=c++11"

%define __cmake_builddir %{_target_platform}

mkdir -p %{%__cmake_builddir}

%cmake

%cmake_build

pushd unix/xserver

sed -i 's@TIGERVNC_BUILDDIR=${TIGERVNC_SRCDIR}@TIGERVNC_BUILDDIR=${TIGERVNC_SRCDIR}/%{_target_platform}@g' hw/vnc/Makefile.am

autoreconf -fiv
%configure \
        --disable-xorg --disable-xnest --disable-xvfb --disable-dmx \
        --disable-xwin --disable-xephyr --disable-kdrive --disable-xwayland \
        --with-pic --disable-static \
        --with-default-font-path="catalogue:%{_sysconfdir}/X11/fontpath.d,built-ins" \
        --with-fontdir=%{_datadir}/X11/fonts \
        --with-xkb-output=%{_localstatedir}/lib/xkb \
        --enable-install-libxf86config \
        --enable-glx --disable-dri --enable-dri2 --disable-dri3 \
        --disable-unit-tests \
        --disable-config-hal \
        --disable-config-udev \
        --with-dri-driver-path=%{_libdir}/dri \
        --without-dtrace \
        --disable-devel-docs \
        --disable-selective-werror

%make_build
popd

# Build icons
pushd %{_target_platform}/media
make
popd

# SELinux
pushd unix/vncserver/selinux
make
popd

%install
%cmake_install
rm -f %{buildroot}%{_docdir}/%{name}-%{version}/{README.rst,LICENCE.TXT}

pushd unix/xserver/hw/vnc
%make_install
popd

# Install systemd unit file
pushd unix/vncserver/selinux
%make_install
popd

# Install systemd unit file
install -m644 %{SOURCE1} %{buildroot}%{_unitdir}/xvnc@.service
install -m644 %{SOURCE2} %{buildroot}%{_unitdir}/xvnc.socket

# Install desktop stuff
mkdir -p %{buildroot}%{_datadir}/icons/hicolor/{16x16,24x24,48x48}/apps

pushd media/icons
for s in 16 24 48; do
install -m644 tigervnc_$s.png %{buildroot}%{_datadir}/icons/hicolor/${s}x$s/apps/tigervnc.png
done
popd

install -m 755 %{SOURCE5} %{buildroot}/%{_bindir}/vncserver

%find_lang %{name} %{name}.lang

# remove unwanted files
rm %{buildroot}%{_docdir}/%{name}/README.rst
rm %{buildroot}%{_datadir}/metainfo/org.tigervnc.vncviewer.metainfo.xml

mkdir -p %{buildroot}%{_sysconfdir}/X11/xorg.conf.d/
install -m 644 %{SOURCE3} %{buildroot}%{_sysconfdir}/X11/xorg.conf.d/10-libvnc.conf

install -m 644 %{SOURCE4} %{buildroot}/%{_docdir}/tigervnc/HOWTO.md

%generate_compatibility_deps

%post server
%systemd_post xvnc@.service
%systemd_post xvnc.socket

%preun server
%systemd_preun xvnc@.service
%systemd_preun xvnc.socket

%postun server
%systemd_postun xvnc@.service
%systemd_postun xvnc.socket

%pre selinux
%selinux_relabel_pre -s %{selinuxtype}

%post selinux
%selinux_modules_install -s %{selinuxtype} %{_datadir}/selinux/packages/%{selinuxtype}/%{modulename}.pp.bz2
%selinux_relabel_post -s %{selinuxtype}

%postun selinux
if [ $1 -eq 0 ]; then
    %selinux_modules_uninstall -s %{selinuxtype} %{modulename}
    %selinux_relabel_post -s %{selinuxtype}
fi


%files -f %{name}.lang
%dir %{abidir}
%{_bindir}/vncviewer
%{_datadir}/applications/*
%{_mandir}/man1/vncviewer.1*

%files doc
%doc README.rst

%files server
%config(noreplace) %{_sysconfdir}/pam.d/tigervnc
%config(noreplace) %{_sysconfdir}/tigervnc/vncserver-config-defaults
%config(noreplace) %{_sysconfdir}/tigervnc/vncserver-config-mandatory
%config(noreplace) %{_sysconfdir}/tigervnc/vncserver.users
%{_unitdir}/vncserver@.service
%{_unitdir}/xvnc@.service
%{_unitdir}/xvnc.socket
%{_bindir}/vncserver
%{_bindir}/x0vncserver
%{_sbindir}/vncsession
%{_libexecdir}/vncserver
%{_libexecdir}/vncsession-start
%{_libexecdir}/vncsession-restore
%{_mandir}/man1/x0vncserver.1*
%{_mandir}/man8/vncserver.8*
%{_mandir}/man8/vncsession.8*
%{_docdir}/tigervnc/HOWTO.md

%files server-minimal
%{_bindir}/vncconfig
%{_bindir}/vncpasswd
%{_bindir}/Xvnc
%{abidir}/Xvnc-option.list
%{_mandir}/man1/Xvnc.1*
%{_mandir}/man1/vncpasswd.1*
%{_mandir}/man1/vncconfig.1*

%files server-module
%{_libdir}/xorg/modules/extensions/libvnc.so
%{abidir}/libvnc.dump
%config(noreplace) %{_sysconfdir}/X11/xorg.conf.d/10-libvnc.conf

%files license
%{_docdir}/tigervnc/LICENCE.TXT

%files icons
%{_datadir}/icons/hicolor/*/apps/*

%files selinux
%{_datadir}/selinux/packages/%{selinuxtype}/%{modulename}.pp.*
%ghost %verify(not md5 size mtime) %{_sharedstatedir}/selinux/%{selinuxtype}/active/modules/200/%{modulename}

%changelog
* Wed Jul 02 2025 tomcruiseqi <10762123+tomcruiseqi@user.noreply.gitee.com> - 1.13.1-3
- Fix CVE-2024-21885

* Wed May 21 2025 zhoujiajia111 <zjl02254423@alibaba-inc.com> - 1.13.1-2
- Rebuild for remove inappropriate build dependencies.

* Wed Mar 08 2023 Ziyang Zhang <ZiyangZhang@linux.alibaba.com> - 1.13.1-1
- Update to 1.13.1(Add libappstream-glib and libXrender-devel build requirement)

* Thu Dec 22 2022 Funda Wang <fundawang@yeah.net> - 1.12.0-5
- Bump release for upgrading from anolis 8

* Wed Nov 02 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.12.0-3
- optimise spec file

* Fri Sep 30 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.12.0-2
- add doc package

* Wed Apr 13 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 1.12.0-1
- init from upstream
